#!/usr/bin/perl -w
#    This script is design to work with Sympa (sympa.org), as a means to sync list data with a ldap entry.
#    Please review the script and modify the ldap attributes to your likeing.
##
#    Copyright (C) 2009  Christopher Pax <pax@njit.edu>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

use strict;
use Net::LDAP;
use Net::LDAP::Entry;
use Net::LDAP::Search;
use DBI;
use Config::General;
use Data::Dumper;


my $conf = new Config::General("/path/to/settings.conf"); 
our %config = $conf->getall;

my $some_dir = "/home/sympa/expl";
opendir(DIR,  $some_dir) || die "can't opendir $some_dir: $!";
#my @dots = grep { /^\./ } readdir(DIR);
my @dots = readdir(DIR);
closedir DIR;


## Connect to date sources ##
our $mysql = DBI->connect("DBI:mysql:database=".$config{'database'}{'database_name'}.";host=".$config{'database'}{'host'},
                         $config{"database"}{"user"}, $config{"database"}{"password"},
                         {'RaiseError' => 1});

our $ldap = Net::LDAP->new( 'ldap.njit.edu' ) or die "$@";
my $mesg = $ldap->bind($config{"ldap"}{"bind_dn"}, password => $config{"ldap"}{"bind_pwd"}) ;
##


foreach my $listname (@dots){
	if($listname =~ /(\.\.|\.)/){
		next;
	}
    # If list name has no DB entry, next
    #~ a list needs at least one owners DB entry to exsist
    my $hasdbres = $mysql->prepare("SELECT * FROM admin_table a where robot_admin= '".$config{"robot"}."' and
	        list_admin = '$listname' and role_admin = 'owner';");
    $hasdbres->execute();
    if(!$hasdbres || $hasdbres->rows <= 0){
		next;
	}
    ###
    # If list has entry in ldap
    my $list_entry = get_list_ldap_entry($listname, $config{"ldap"}{"base_dn"});
    if($list_entry){
        my $is_sync = 1;# initially assumed to be in sync
        my %new_attrs = (); # to be used if out of sync
		# check if cn, sn, and mail are sync
        if($list_entry->get_value("cn") ne $listname or
        $list_entry->get_value("sn") ne $listname or
        $list_entry->get_value("mail") ne $listname."\@".$config{"robot"}){
            $is_sync = 0; #no, its not in sync
            $new_attrs{"cn"} = $listname;
            $new_attrs{"sn"} = $listname;
            $new_attrs{"mail"} = $listname."\@".$config{"robot"};
        }
        
        # check if owners are in sync
        my @ldap_owners = $list_entry->get_value("owner");
        my @db_owners = get_owners($listname);
        
        my $owner_add_attr = 0;
        foreach my $db_owner (@db_owners){
            my @match = grep(/$db_owner/i, @ldap_owners); 
            if(!@match){
                $is_sync = 0; #no longer in sync
                $owner_add_attr = 1; #will update %new_attrs later
                #push(@db_owners, $ldap_owner);
            }
        }
        if($owner_add_attr){
		#print Dumper(@db_owners);
		$new_attrs{"owner"} = \@db_owners;
	}
        # If out-of-sync
        if(!$is_sync){
            #modify ldap entry with values from sympa
	    print "$listname out of sync, syncing now\n";
	    #print Dumper(\%new_attrs)."\n";
	    update_ldap_entry("uid=$listname,".$config{"ldap"}{"base_dn"}, \%new_attrs);
	    print "##################\n";
        }
    }
    # If list does not have entry in ldap
    else{
        #create entry for ldap, and add it  
        my @db_owners = get_owners($listname);
        my %attrs = (
            objectClass => ["top","person", "organizationalPerson", "inetorgperson", "groupOfNames"],
            cn => [$listname],
            sn => [$listname],
            uid => [$listname],
            preferredDeliveryMethod => ['catchall'],
            owner => \@db_owners,   
        );
	my $entry = Net::LDAP::Entry->new;
	$entry->dn("uid=$listname,".$config{"ldap"}{"base_dn"});
	$entry->add("mail", $listname."\@".$config{"robot"});
	foreach my $hash_key (keys %attrs) {
		foreach my $hash_value (@{$attrs{$hash_key}}) {
                	$entry->add($hash_key, $hash_value);
            	}
        }

        my $result = $ldap->add($entry);
	#$result->code && die "failed to add entry: ", $result->error;

	print "adding $listname to ldap\n";            
        #print Dumper(\%attrs)."\n";
	print "########################\n";
    }
}
##




sub update_ldap_entry {
    my $ldapdn = shift;
    my %attrs = %{$_[0]};
    
    #print Dumper(%attrs);

    my $entry = Net::LDAP::Entry->new;
    $entry->dn($ldapdn);
    
    #foreach my $hash_key (keys %attrs) {
    #    foreach my $hash_value (@{$attrs{$hash_key}}) {
    #        $entry->add($hash_key, $hash_value);
    #    }
    #}
    my$result = $ldap->modify($entry,replace => \%attrs);#$ldapdn, replace => %attrs);
    #my $result = $ldap->add($entry);
    $result->code && die "failed to modify entry: ", $result->error ;
    $mesg = $ldap->unbind;   # take down session
}

sub get_owners{
	my $listname = shift;
	
	my $query = " 
        	SELECT `user_admin`, `role_admin`, `profile_admin` FROM admin_table a where
	        robot_admin= '".$config{"robot"}."' and
	        list_admin = '{list_name}' and
	        role_admin = 'owner';
	";
	$query =~ s/{list_name}/$listname/;
	my $dbresults = $mysql->prepare($query);
	$dbresults->execute();
	my @owners = ();
	while (my $row = $dbresults->fetchrow_hashref()) {
		push(@owners, $row->{'user_admin'});
	}
	if(!$dbresults || $dbresults->rows <= 0){
		return ();
	}
	$dbresults->finish();
	return @owners;
}




sub get_list_ldap_entry{
	my $listname = shift;
	my $dn = shift;
	if(defined $dn){
		$dn = $config{"ldap"}{"base_dn"};
	}

	my $results = $ldap->search(base => $dn, filter=> "uid=$listname", sizelimit => 1,scope=>"sub");
	if($results->code){
		#print $results->error."\n";
		return 0;
	}
	if($results->count() <= 0){
		return 0;
	}
	foreach my $entry ($results->entries) { 
		return $entry; 
	}
	return 0;
}
